package visualization3;

import java.util.ArrayList;
import java.util.List;

/**
 * 階層的クラスタリングを実行するクラスです。
 */

public class ClusterBuilder {
    private DistanceEvaluator distanceEvaluator;

    /**
     * 指定された距離関数に基づいてビルダを作成します。
     * @param distanceEvaluator 距離関数
     */
    public ClusterBuilder(DistanceEvaluator distanceEvaluator) {
        this.distanceEvaluator = distanceEvaluator;
    }

    /**
     * クラスタリングを実行し、ノード階層を構築します。
     * @param nodes クラスタリング対象のノードリスト
     * @return クラスタリング結果の最上位ノード
     */
    public Node build(List<? extends Node> nodes) {
        // ノードが1つに集約されるまで繰り返す
        while (nodes.size() > 1) {
            Node merge1 = null;
            Node merge2 = null;
            double minDist = Double.MAX_VALUE;
            // 距離が最小となるノード対を探す
            for (int i = 0; i < nodes.size(); i++) {
                Node n1 = nodes.get(i);
                for (int j = i + 1; j < nodes.size(); j++) {
                    Node n2 = nodes.get(j);
                    double dist = distanceEvaluator.distance(n1, n2);
                    if (dist < minDist) {
                        merge1 = n1;
                        merge2 = n2;
                        minDist = dist;
                    }
                }
            }
            // 次ステップ用のノードリストを作成
            List<Node> nextNodes = new ArrayList<Node>();
            for (Node node : nodes) {
                // 統合対象にならなかったノードを追加
                if (node != merge1 && node != merge2) {
                    nextNodes.add(node);
                }
            }
            // 統合対象のノード対をクラスタ化して追加
            nextNodes.add(new Cluster(merge1, merge2));
            nodes = nextNodes;
        }
        return nodes.get(0);
    }
}
