package visualization3;

import java.util.List;

/**
 * Ward法に基づく距離関数の実装です。
 */

public class WardDistanceEvaluator implements DistanceEvaluator {
    public double distance(Node n1, Node n2) {
        double s12 = sumCentroidDistanceSq(new Cluster(n1, n2));
        double s1 = sumCentroidDistanceSq(n1);
        double s2 = sumCentroidDistanceSq(n2);
        return s12 - (s1 + s2);
    }

    private double sumCentroidDistanceSq(Node n) {
        List<MultiVector> vectors = n.getVectors();
        // ノードに含まれる全ベクトルの平均値を求め、クラスタの重心とする
        MultiVector center = new MultiVector(vectors.get(0).dimension());
        for (MultiVector vector : vectors) {
            center.add(vector);
        }
        center.divide(vectors.size());

        // ノードに含まれる各ベクトルと重心の距離の二乗の総和を計算
        double sum = 0;
        for (MultiVector vector : vectors) {
            sum += vector.distanceSq(center);
        }
        return sum;
    }
}
